/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.uinfo.salix;

import java.util.HashSet;
import java.util.Set;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.common.AbstractCalculationResult;
import org.dive4elements.river.artifacts.uinfo.commons.UInfoResultType;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.exports.process.DefaultProcessor;
import org.dive4elements.river.exports.process.WOutProcessor;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.themes.ThemeDocument;
import org.dive4elements.river.utils.RiverUtils;
import org.jfree.data.xy.XYSeries;

/**
 * Processor to generate the facets and data series of salix line
 *
 * @author Matthias Schäfer
 *
 */
public final class SalixLineProcessor extends DefaultProcessor {

    private static final String FACET_SALIX_LINE = "uinfo_facet_salix";

    private static final String FACET_SALIX_MNWMW = "uinfo_facet_mnwmw";

    private static final Set<String> HANDLED_FACET_TYPES = new HashSet<>();

    static {
        HANDLED_FACET_TYPES.add(FACET_SALIX_LINE);
        HANDLED_FACET_TYPES.add(FACET_SALIX_MNWMW);
    }

    public static Facet createSalixLineFacet(final CallContext context, final String hash, final String id, final AbstractCalculationResult result,
            final int facetIndex, final int resultIndex, final String description) {

        return new SalixLineResultFacet(FACET_SALIX_LINE, description);
    }

    public static Facet createSalixMnwMwFacet(final CallContext context, final String hash, final String id, final AbstractCalculationResult result,
            final int facetIndex, final int resultIndex, final String description) {

        return new SalixMnwMwResultFacet(FACET_SALIX_MNWMW, description);
    }

    @Override
    public final String getAxisLabel(final DiagramGenerator generator) {

        final D4EArtifact flys = (D4EArtifact) generator.getMaster();
        final String unit = RiverUtils.getRiver(flys).getWstUnit().getName();
        return generator.msg(WOutProcessor.I18N_AXIS_LABEL, WOutProcessor.I18N_AXIS_LABEL_DEFAULT, unit);
    }

    @Override
    public final boolean canHandle(final String facettype) {
        return HANDLED_FACET_TYPES.contains(facettype);
    }

    @Override
    public void doOut(final DiagramGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible) {

        // log.debug("Processing facet: " + bundle.getFacetName());
        final CallContext context = generator.getContext();
        final Object data = bundle.getData(context);

        final XYSeries series = new StyledXYSeries(bundle.getFacetDescription(), theme);

        if (bundle.getFacetName().equals(FACET_SALIX_LINE)) {
            StyledSeriesBuilder.addPoints(series, ((SalixLineCalculationNoScenarioResult) data).getStationPoints(UInfoResultType.salixline), true);
            generator.addAxisSeries(series, this.axisName, visible);
            return;
        }

        if (bundle.getFacetName().equals(FACET_SALIX_MNWMW)) {
            StyledSeriesBuilder.addPoints(series, ((SalixLineCalculationNoScenarioResult) data).getStationPoints(UInfoResultType.salix_delta_mw), true);
            generator.addAxisSeries(series, this.axisName, visible);
            return;
        }
    }
}